from django.db import models

from wagtail.core.models import Page
from wagtail.core.fields import RichTextField
from wagtail.admin.edit_handlers import FieldPanel, InlinePanel
from wagtail.search import index
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.core.fields import StreamField
from wagtail.core import blocks
from wagtail.admin.edit_handlers import FieldPanel, StreamFieldPanel
from wagtail.images.blocks import ImageChooserBlock

class ManualTopicBlock(blocks.StructBlock):
    slug = blocks.CharBlock()
    heading = blocks.CharBlock()
    paragraph = blocks.RichTextBlock()
    image = ImageChooserBlock(required=False)
    
    class Meta:
        template = 'pages/blocks/manual_topic_block.html'


class IndexPage(Page):
    intro = RichTextField(blank=True)

    content_panels = Page.content_panels + [
        FieldPanel('intro', classname="full")
    ]

class ManualPage(Page):
    """
    query in django views using:
    p = ManualPage.objects.get(id=7)
    p.body[0].__dict__

    """    
    intro = RichTextField(blank=True)
    body = StreamField([
        ('topic', ManualTopicBlock()),
    ])
    content_panels = Page.content_panels + [
        FieldPanel('intro'),        
        StreamFieldPanel('body'),
    ]

    


class TeamMemberIndexPage(Page):
    intro = RichTextField(blank=True)

    content_panels = Page.content_panels + [
        FieldPanel('intro', classname="full")
    ]

class TeamMemberPage(Page):        
    text = RichTextField(blank=True)
    link = models.URLField(blank=True)
    image = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.PROTECT, related_name='+', blank=True, null=True
    )

    search_fields = Page.search_fields + [               
        index.SearchField('text'),
    ]

    content_panels = Page.content_panels + [            
        FieldPanel('text', classname="full"),
        FieldPanel('link'),
        ImageChooserPanel('image'),
    ]
