from __future__ import unicode_literals

import logging
logger = logging.getLogger('core')

from django.db import models

from taggit.managers import TaggableManager
from taggit.models import TagBase, GenericTaggedItemBase, GenericUUIDTaggedItemBase

class Topic(models.Model):
    title = models.CharField(max_length=200)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.title

class Keyword(models.Model):
    title = models.CharField(max_length=200)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.title

class EnTagged(TagBase):
    class Meta:
        verbose_name = "English Tag"
        verbose_name_plural = "English Tags"

class EnglishTags(GenericTaggedItemBase):
    tag = models.ForeignKey(EnTagged, on_delete=models.CASCADE)

class AffiliationTagged(TagBase):
    class Meta:
        verbose_name = "Affiliation Tag"
        verbose_name_plural = "Affiliation Tags"

class AffiliationTags(GenericTaggedItemBase):
    tag = models.ForeignKey(AffiliationTagged, related_name="affiliation_tags", on_delete=models.CASCADE)
    def __str__(self):
        return self.name

class KeywordTagged(TagBase):
    class Meta:
        verbose_name = "Keyword Tag"
        verbose_name_plural = "Keyword Tags"

class KeywordTags(GenericTaggedItemBase):
    tag = models.ForeignKey('KeywordTagged', related_name="keywordtags_keywordtagged", on_delete=models.CASCADE)
    def __str__(self):
        return self.tag.name

class LanguageTagged(TagBase):
    class Meta:
        verbose_name = "Language Tag"
        verbose_name_plural = "Language Tags"

class LanguageTags(GenericTaggedItemBase):
    tag = models.ForeignKey('LanguageTagged', related_name="languagetags_languagetagged", on_delete=models.CASCADE)
    def __str__(self):
        return self.tag.name


class Link(models.Model):
    url = models.URLField()
    doi = models.CharField(max_length=200, blank=True)
    description = models.CharField(max_length=400, blank=True)

    def lookup_doi(self):
        service, doi_dict = lookup_doi(self.doi)
        print( doi_dict['author'] )

    def __str__(self):
        return self.url



