import logging
logger = logging.getLogger('core')

from django.shortcuts import get_object_or_404, render, redirect
from django.core.exceptions import PermissionDenied

from guardian.shortcuts import assign_perm, get_users_with_perms, get_perms, remove_perm, get_objects_for_user, get_groups_with_perms
from guardian.decorators import permission_required_or_403
from guardian.core import ObjectPermissionChecker

# Create your views here.

def check_perms(permission, user, dd_obj):
    logger.debug('start checks_perms')    
    checker = ObjectPermissionChecker(user)        
    
    try:        
        if dd_obj.owner == user:
            logger.debug('is owner')            
            return True
    except: 
        logger.debug('is not owner')               
        pass
    
    if checker.has_perm(permission, dd_obj):
        logger.debug('%s has perm %s on %s' % (user, permission, dd_obj) )           
        return True    
    else:        
        logger.debug('%s lacks perm %s on %s' % (user, permission, dd_obj) )            
        return False

def check_perms_403(permission, user, dd_obj):
    if not check_perms(permission, user, dd_obj):
        raise PermissionDenied()


def get_oep_parameters(type, outer_panel, add_text):
    return {
        'type':type,
        'title':'{}s'.format( type.capitalize(), ),
        'edit_url':'{}_add'.format(type, ),
        'list_url':'get_user_{}s'.format(type, ),
        'outer_panel':outer_panel,
        'add_text': add_text,
    }


def handler404(request):
    # was: render_to_response, which is deprecated
    response = render('404.html', {},
                                  context_instance=RequestContext(request))
    response.status_code = 404
    return response
