# -*- coding: utf-8 -*-
import sys

import logging
logger = logging.getLogger('ddusers')
#logger = logging.getLogger(__name__)

from rest_framework import mixins
from rest_framework import generics

from django.core.exceptions import PermissionDenied

from django.shortcuts import get_object_or_404, render, redirect
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger


from django.forms import modelformset_factory, inlineformset_factory
from django.http import HttpResponseRedirect, Http404, HttpResponse, FileResponse, HttpResponseForbidden

from io import StringIO, BytesIO
from zipfile import ZipFile

from django.urls import reverse

from django.contrib.auth.models import Group
from django.forms import formset_factory
from django.forms.models import model_to_dict

from core.views import check_perms, get_oep_parameters, handler404

from ddusers.models import User #user
from ddusers.models import Institution #user
from ddusers.models import Project #user

from dddatasets.models import DataSet, DataSetManagementObject #data
from dddatasets.models import DataFile #data

from ddusers.models import Country
from ddusers.models import City
from ddusers.models import Publication
from ddusers.models import Affiliation #user

from core.models import AffiliationTagged
from core.models import Topic #data
from core.models import KeywordTags
from core.models import KeywordTagged
from core.models import LanguageTagged

from dddatasets.models import AnalysisMethodsTagged
from dddatasets.models import CollectionMethodsTagged
from dddatasets.models import Category
from dddatasets.models import Documentation
from dddatasets.models import DataSetPublication


from ddusers.serializers import UserSerializer
from ddusers.serializers import InstitutionSerializer
from ddusers.serializers import ProjectSerializer
from ddusers.serializers import DataSetSerializer
from ddusers.serializers import TopicSerializer

from ddcomments.models import Comment
from ddcomments.forms import CommentForm

from django.contrib.contenttypes.models import ContentType

from taggit.models import Tag

import tempfile
from weasyprint import HTML

from django_downloadview import ObjectDownloadView

from django.core.files.uploadedfile import SimpleUploadedFile
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test

from ddusers.forms import UserForm, TopicForm, ProjectForm, IcUserdataForm, CountryForm,  InstitutionForm, AffiliationForm, PublicationForm

from dddatasets.forms import DataSetForm, DataSetDescForm, DataSetImageUploadForm, DataFileUploadForm, DataSetPublicationForm

from django.contrib import messages
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.forms import PasswordChangeForm
from django.shortcuts import render, redirect

from django.contrib.contenttypes.models import ContentType

from django.conf import settings
from guardian.shortcuts import assign_perm, get_users_with_perms, get_perms, remove_perm, get_objects_for_user, get_groups_with_perms
from guardian.decorators import permission_required_or_403
from guardian.core import ObjectPermissionChecker

from actstream.models import actor_stream, user_stream, model_stream, Action, Follow, following, followers
from dddatasets.models import DataSetManagementObject



def start_page(request):
    if request.user.is_authenticated:
        user = User.objects.get(id=request.user.id)
        actorstream=user.actor_actions.all() # full with title
        actfollowing = following(request.user, User)
        dsmofollowing = Follow.objects.filter(user=user).filter(content_type_id=44)
        actfollowers = followers(request.user)

        userct = ContentType.objects.get_for_model(User)
        dsmoct = ContentType.objects.get_for_model(DataSetManagementObject)

        userstream = Action.objects.filter(public=True).filter(actor_content_type_id=userct.id).exclude(actor_object_id=user.id).filter(target_content_type_id=dsmoct.id) #.filter(target_object_id='af514eeb-3763-4fc8-9cb5-6023eb38dfcc')  #None#user_stream(request.user)
        

        return render(request, 'index.html', {'user':user, 'userstream':userstream, 'actorstream':actorstream, 'dsmofollowing': dsmofollowing, 'followers':actfollowers, 'following':actfollowing, })
    else:        
        return render(request, 'index.html', {})

"""
User: change password
"""
@login_required
def change_password(request):
    if request.method == 'POST':
        form = PasswordChangeForm(request.user, request.POST)
        if form.is_valid():
            user = form.save()
            update_session_auth_hash(request, user)  # Important!
            messages.success(request, 'Your password was successfully updated!')
            #return redirect('data/change_password')
            return render(request, 'auth/change_password.html', {'form': form, 'user': user, 'updated': True, 'err':None, })
        else:
            messages.error(request, 'Please correct the error below.')
    else:
        form = PasswordChangeForm(request.user)
    return render(request, 'auth/change_password.html', {
        'form': form
    })


@login_required
def get_user_countries_ic(request, pk=None):
    user = User.objects.get(id=request.user.id)
    all_countries = Country.objects.filter(public=True)
    countries = list()
    if pk:
        country = Country.objects.get(id=pk)
        if country in user.get_countries():
            user.countries.remove( country )
        else:
            user.countries.add( country )
        user.save()

    for ac in all_countries:
        if ac in user.get_countries():
            countries.append( (ac, True) )
        else:
            countries.append( (ac, False) )
    return render(request, 'get_user_countries.html', {'user': user, 'countries':countries,})



@login_required
def get_user_projects(request):
    user = User.objects.get(id=request.user.id)
    objects = user.get_projects()
    object_type = 'project'
    object_text = 'Add new Discuss Data related project...'
    edit_url = 'project_edit'
    add_url = 'project_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})

@login_required
def get_user_datasets(request):
    logger.debug("get_user_datasets")
    user = User.objects.get(id=request.user.id)
    objects_own = DataSet.objects.filter(owner=request.user)
    objects_view = get_objects_for_user(user, 'view_dataset', DataSet).exclude(owner=request.user)
    objects_edit = get_objects_for_user(user, 'edit_dataset', DataSet).exclude(owner=request.user)
    objects_admin = get_objects_for_user(user, 'admin_dataset', DataSet).exclude(owner=request.user)     
    logger.debug(objects_own)
    logger.debug(objects_view)
    object_type = 'dataset'
    edit_url = 'dataset_edit'
    add_url = 'dataset_add'
    delete_url ='ic_ds_remove'
    
    return render(request, 'data/get_user_datasets.html', {'user': user, 'objects_edit':objects_edit, 'objects_own':objects_own, 
    'objects_view':objects_view, 'edit_url': edit_url, 'add_url': add_url,  'delete_url': delete_url, 'object_type': object_type})

@login_required
def get_user_publications(request):
    user = User.objects.get(id=request.user.id)
    objects = user.publications.all()
    object_type = 'publication'
    object_text = 'Add new Discuss Data related publication...'
    edit_url = 'publication_edit'
    add_url = 'publication_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})
    #return render(request, 'data/get_user_publications.html', {'user': user, })

@login_required
def publication_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    publication = Publication.objects.get(id=pk)
    user.publications.remove(publication)
    objects = user.publications.all()
    object_type = 'publication'
    object_text = 'Add new Discuss Data related publication...'
    edit_url = 'publication_edit'
    add_url = 'publication_add'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url,  'object_type': object_type, 'object_text': object_text,})

@login_required
def project_remove(request, uuid):
    user = User.objects.get(id=request.user.id)
    project = Project.objects.get(uuid=uuid)
    objects = Project.objects.filter(user=user)
    object_type = 'project'
    object_text = 'Add new Discuss Data related project...'
    edit_url = 'project_edit'
    add_url = 'project_add'
    delete_url = 'project_remove'
    if user == project.user:
        project.delete()
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url, 'delete_url': delete_url, 'object_type': object_type, 'object_text': object_text,})

@login_required
def country_ic_add(request):
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        add = request.POST.get('country-tag','')

        #print( "REMOVE", remove )
        #print( "ADD", add )
        # set user to authenticated user
        #if form.is_valid():
        #    # get tags from form
        #    interests = form.cleaned_data["interests"]
        #    # clear all tags from user
        #    user.interests.clear()
        #    # add tags from form to user
        if len(add) > 0:
            user.countries.add(add)

        return render(request, 'get_user_countries.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'get_user_countries.html', {'user': user,})

@login_required
def affiliation_add(request):
    user = User.objects.get(id=request.user.id)
    if request.method == 'POST':
        add = request.POST.get('affiliation-tag','')
        if len(add) > 0:
            user.affiliations.add(add)

        return render(request, 'get_user_affiliations.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'get_user_affiliations.html', {'user': user,})

#@login_required
#def affiliation_remove(request, pk):
#    user = User.objects.get(id=request.user.id)
#    tag = AffiliationTagged.objects.get(id=pk)
#    user.affiliations.remove(tag)
#    return render(request, 'get_user_affiliations.html', {'user': user, })

#def get_affiliations(request):
#    qs = AffiliationTagged.objects.all()
    #query = None
    #if query:
    #    return qs.filter(name__istartswith=query)
    #else:
    #return render(request, 'get_tags.html', {'qs': qs, })

@login_required
def tag_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    tag = KeywordTagged.objects.get(id=pk)
    user.interests.remove(tag)
    return render(request, 'user/get_user_interests.html', {'user': user, })

@login_required
def tag_add(request):
    user = User.objects.get(id=request.user.id)
    print(user)
    if request.method == 'POST':
        add = request.POST.get('interest-tag','').lower()                
        if len(add) > 0:
            user.interests.add(add)

        return render(request, 'user/get_user_interests.html', {'user': user, 'updated': True, 'err':None, })
    else:
        return render(request, 'user/get_user_interests.html', {'user': user,})

def get_object_tags(request, object_type):
    print("object_type")
    print(object_type)
    if object_type == 'language':
        qs = LanguageTagged.objects.all()
    if object_type == 'keyword':
        qs = KeywordTagged.objects.all()
    if object_type == 'methods_of_data_collection':
        qs = CollectionMethodsTagged.objects.all()
    if object_type == 'methods_of_data_analysis':
        qs = AnalysisMethodsTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_language_tags(request):
    qs = LanguageTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_users_all(request):
    qs = User.objects.all()
    return render(request, 'get_tags.html', {'qs': qs, })

def get_keyword_tags(request):
    qs = KeywordTagged.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def get_institutions(request):
    qs = Institution.objects.all()
    query = None
    if query:
        return qs.filter(name__istartswith=query)
    else:
        return render(request, 'get_tags.html', {'qs': qs, })

def dataset_get_users(ds):
    return get_users_with_perms(ds)

@login_required
def get_user_interests(request):
    user = User.objects.get(id=request.user.id)
    return render(request, 'user/get_user_interests.html', {'user': user, })

@login_required
def get_user_profile(request):
    user = User.objects.get(id=request.user.id)
    return render(request, 'get_user_profile.html', {'user': user, })


@login_required
def userdata_edit(request):
    err = None
    updated = False
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = IcUserdataForm(request.POST, request.FILES, instance=user)
        if form.is_valid():
            user = form.save()
            user.save()
            updated = True
        else:
            err = form.errors.as_data()
    else:
        user = User.objects.get(id=request.user.id)
        form = IcUserdataForm(instance=user)
    return render(request, 'project_edit.html', {'form': form, 'user': user, 'updated': updated, 'err': err, 'target': 'User Profile',})

@login_required
def publication_edit(request, pk=None):
    if pk:
        # existing publication?
        publication = Publication.objects.get(id=pk)
        new = False
    else:
        # new publication
        publication = Publication()
        new = True
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(request.POST, instance=publication)
        if form.is_valid():
            publication = form.save()
            user.publications.add(publication)
            return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'updated': True, 'err':None, 'new':new, })
        else:
            return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'updated': False, 'err':  form.errors.as_data(), 'new':new,})
    else:
        user = User.objects.get(id=request.user.id)
        form = PublicationForm(instance=publication)
    return render(request, 'project_edit.html', {'target': 'publication', 'form': form, 'user': user, 'err':None, 'new':new,})




        

        
    


# def dataset_user_list(request, pk):
#     ds = DataSet.objects.get(id=pk)
#     oep = {
#         'div_id':'dataset-users',
#         'input_id':'dataset-user',
#         #'objects_get': ds.get_users,
#         'url_search_object':'user_search',
#         'url_object_add':'tag_add',
#         'url_object_remove':'tag_remove',
#         'text':'Dataset Users',
#     }
#     if 'view_dataset' in get_perms(request.user, ds):
#         oep['objects_get'] = get_users_with_perms(ds)
#         #return render(request, 'data/perms/users_perms.html', {'ds': ds, 'oep': oep})
#         return render(request, 'user/add_user_to_object_incl.html', {'ds': ds, 'oep': oep})
    





        
    

def user_search(request):
    oep = {
        'div_id':'dataset-users',
        'input_id':'dataset-user',
        'url_search_object':'user_search',
        'url_object_add':'dataset_user_add',
        'url_object_remove':'dataset_user_remove',
        'text':'Dataset Users',
    }
    users = None
    if request.POST:
        try:
            q = request.POST['search']
            
            groupid = request.POST['groupid']                       
            oep['dsid'] = request.POST['dsid']
            
            users = User.objects.filter(last_name__icontains=q) | User.objects.filter(first_name__icontains=q)
            
        except:
            pass
        logger.debug('search query: {}, groupid: {}, ds id: {}, users: {}'.format(q, groupid, oep['dsid'], users))
    return render(request, 'user/user_list.html', {'users':users, 'query':q, 'groupid':groupid, 'oep':oep, })



    
#def dataset_user_list(request, pk):
#    ds = DataSet.objects.get(id=pk)
#    oep = {
#        'div_id':'dataset-users',
#        'input_id':'dataset-user',
#        'objects_get': ds.get_users,
#        'url_search_object':'user_search',
#        'url_object_add':'tag_add',
#        'url_object_remove':'tag_remove',
#        'text':'Dataset Users',
#    }
#    return render(request, 'user/add_user_to_object_incl.html', {'ds': ds, 'oep': oep,})





def add_user_to_group(request, pk, group):
    pass



#@login_required
#def ic_ds_language_tags_add(request, ds_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.owner != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    if request.method == 'POST':
#        add = request.POST.get('language-tag','')
#        if len(add) > 0:
#            ds.languages_of_data.add(add)
#    else:
#        pass
#    remove_url = 'ic_ds_language_tags_remove'
#    get_url = 'get_language_tags'
#    add_url = 'ic_ds_language_tags_add'
#    #return render(request, 'data/ic_ds_keyword_tags.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), })
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds': ds, 'tags': ds.get_language_tags(), 'object':'language',  'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url,})



#@login_required
#def ic_ds_keyword_tags_add(request, ds_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.owner != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    if request.method == 'POST':
#        add = request.POST.get('keyword-tag','')
#
#        if len(add) > 0:
#            ds.keyword_tags.add(add)
#    else:
#        pass
#    #return render(request, 'data/ic_ds_keyword_tags.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), })
#    remove_url = 'ic_ds_keyword_tags_remove'
#    get_url = 'get_keyword_tags'
#    add_url = 'ic_ds_keyword_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds': ds, 'tags': ds.get_keyword_tags(), 'object':'keyword', 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })

#@login_required
#def ic_ds_keyword_tags_remove(request, ds_pk, keyword_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.owner != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    tag = KeywordTagged.objects.get(id=keyword_pk)
#    ds.keyword_tags.remove(tag)
#    remove_url = 'ic_ds_keyword_tags_remove'
#    get_url = 'get_keyword_tags'
#    add_url = 'ic_ds_keyword_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds':ds, 'tags': ds.get_keyword_tags(), 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })

#@login_required
#def ic_ds_language_tags_remove(request, ds_pk, language_pk):
#    ds = DataSet.objects.get(id=ds_pk)
#    if ds.owner != request.user:
#        raise Http404("No right to access this page")
#    user = User.objects.get(id=request.user.id)
#    tag = LanguageTagged.objects.get(id=language_pk)
#    ds.languages_of_data.remove(tag)
#    remove_url = 'ic_ds_language_tags_remove'
#    get_url = 'get_language_tags'
#    add_url = 'ic_ds_language_tags_add'
#    return render(request, 'data/tag_add_control.html', {'user': user, 'ds':ds, 'tags': ds.get_language_tags(), 'remove_url': remove_url, 'get_url': get_url, 'add_url':add_url, })



#@login_required
#def dataset_edit(request, pk=None):
#    if pk:
#        template = 'data/dataset_edit.html'
#        # existing dataset?
#        print("User", request.user)
#        ds = DataSet.objects.get(id=pk)
#        form = DataSetImageUploadForm(request.POST, request.FILES)
#        print("DS User", ds.owner)
#        ds_id = pk
#        if ds.owner != request.user:
#            raise Http404("No right to access this page")
#    else:
#        # new project
#        template = 'data/dataset_add.html'
#        ds = DataSet()
#        form = DataSetImageUploadForm(request.POST, request.FILES)
#        ds_id = ""
#        #ds_desc.dataset = dataset
#    return render(request, template, {'ds': ds, 'ds_id': ds_id, 'err':None, 'form':form,})




@login_required
def get_user_affiliations(request):
    user = User.objects.get(id=request.user.id)
    objects = user.get_affiliations()
    object_type = 'affiliation'
    object_text = 'Add new affiliation...'
    edit_url = 'affiliation_edit'
    add_url = 'affiliation_add'
    delete_url = 'affiliation_remove'
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url, 'add_url': add_url, 'delete_url': delete_url, 'object_type': object_type, 'object_text': object_text})

@login_required
def affiliation_remove(request, pk):
    user = User.objects.get(id=request.user.id)
    affiliation = Affiliation.objects.get(id=pk)
    objects = Affiliation.objects.filter(user=user)
    object_type = 'affiliation'
    object_text = 'Add new affiliation...'
    edit_url = 'affiliation_edit'
    add_url = 'affiliation_add'
    delete_url = 'affiliation_remove'
    if user == affiliation.user:
        affiliation.delete()
    return render(request, 'get_user_objects.html', {'user': user, 'objects':objects, 'edit_url': edit_url,
            'add_url': add_url, 'delete_url': delete_url, 'object_type': object_type, 'object_text': object_text})

@login_required
def affiliation_edit(request, pk=None):
    form = AffiliationForm(prefix='aff')
    if pk:
        # existing affiliation?
        affiliation = Affiliation.objects.get(id=pk)
        institution = affiliation.institution
        if affiliation.user != request.user:
            raise Http404("No right to access this page")
        new = False
    else:
        # new project
        affiliation = Affiliation()
        new = True

    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = AffiliationForm(request.POST, request.FILES, prefix='aff', instance=affiliation)
        if form.is_valid(): #and sub_form.is_valid():
            affiliation = form.save(commit=False)
            affiliation.user = user
            affiliation.save()
            updated = True
            form_err = None
        else:
            updated = False
            form_err = form.errors.as_data()

    else:
        user = User.objects.get(id=request.user.id)
        form = AffiliationForm(prefix='aff', instance=affiliation)
        updated = False
        form_err = None
    return render(request, 'project_edit.html', {'target': 'affiliation', 'form': form, 'user': user, 'form_err':form_err, 'updated':updated, 'new':new,})
    #return render(request, 'affiliation_edit.html', {'target': 'affiliation', 'form': form, 'sub_form':sub_form, 'user': user, 'form_err':form_err, 'sub_form_err':sub_form_err, 'updated':updated})

@login_required
def project_edit(request, uuid=None):
    if uuid:
        # existing project?
        project = Project.objects.get(uuid=uuid)
        if project.user != request.user:
            raise Http404("No right to access this page")
        new = False
    else:
        # new project
        project = Project()
        new = True

    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        form = ProjectForm(request.POST, request.FILES)
        if form.is_valid():
            project.name = form.cleaned_data["name"]
            project.url = form.cleaned_data["url"]
            project.user = user
            project.save()

            return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'updated': True, 'err':None, 'new':new, })
        else:
            return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'updated': False, 'err':  form.errors.as_data(), 'new':new,})
    else:
        user = User.objects.get(id=request.user.id)
        form = ProjectForm(instance=project)
    return render(request, 'project_edit.html', {'target': 'project', 'form': form, 'user': user, 'err':None, 'new':new,})
    #else:
    #    error_msg = "You don't have the rights to edit this dataset"
    #    return render(request, 'data/dataset_edit.html', {'error_msg': error_msg, 'user': user})

def check_user_group(user, group_name):
    return user.groups.filter(name=group_name).exists()



#@login_required
#def dataset_edit(request, pk):
#    user = User.objects.get(id=request.user.id)
#    dataset = DataSet.objects.get(id=pk)
#    if user:
#        if request.method == 'POST':
#            # set user to authenticated user
#            user = User.objects.get(id=request.user.id)
#            dataset_form = DataSetForm(request.POST, request.FILES)
#            if dataset_form.is_valid():
#                print( dataset_form)
#            else:
#                print( "not valid")
#                print( dataset_form.errors.as_data())
#        else:
#            dataset_form = DataSetForm(instance=dataset)
#        return render(request, 'data/dataset_edit.html', {'dataset_form': dataset_form, 'user': user})
#    else:
#        error_msg = "You don't have the rights to edit this dataset"
#        return render(request, 'data/dataset_edit.html', {'error_msg': error_msg, 'user': user})
#

#@login_required
#def topic_edit(request):
#    if request.method == 'POST':
#        topic_form = TopicForm(request.POST)
#        if topic_form.is_valid():
#            title = topic_form.cleaned_data["topic"]
#            description = topic_form.cleaned_data["description"]
#            try:
#                topic = Topic.objects.get(title=title)
#            except:
#                topic = Topic()
#                topic.title = title
#                topic.description = description
#                topic.save()
#            return HttpResponseRedirect('/data/user/' + str(user.id))
#        else:
#            print "not valid"
#            print topic_form.errors.as_data()
#


@login_required
def topic_add(request):
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        topic_form = TopicForm(request.POST)
        if topic_form.is_valid():
            title_en = topic_form.cleaned_data["title_en"]
            title_ru = topic_form.cleaned_data["title_ru"]
            description_en = topic_form.cleaned_data["description_en"]
            description_ru = topic_form.cleaned_data["description_ru"]
            try:
                topic = Topic.objects.get(title=title_en)
            except:
                topic = Topic()
            topic.title_en = title_en
            topic.title_ru = title_ru
            topic.description_en = description_en
            topic.description_ru = description_ru
            topic.save()
            user.topics.add(topic)
            user.save()
            saved = True
            return render(request, 'topic_add.html', {'saved':saved, 'topic':topic})
        else:
            return HttpResponse("Form not valid")
    else:
        user = User.objects.get(id=request.user.id)
        topic_form = TopicForm()
    #return HttpResponse("No post data")
    return render(request, 'topic_add.html', {'topic_form': topic_form, 'user': user})

@login_required
def get_topics(request):
    topics = Topic.objects.all()
    return render(request, 'topic_list_ic.html', {'topics':topics,})

@login_required
def user_datasets(request):
    user = User.objects.get(id=request.user.id)
    logger.debug("user_datasets")
    return render(request, 'user_datasets.html', {'user': user})



@login_required
def user_edit(request):
    oep_publications = get_oep_parameters('publication', True, 'Add new Discuss Data related publication...')
    oep_projects = get_oep_parameters('project', True, 'Add new Discuss Data related project...')
    oep_affiliations = get_oep_parameters('affiliation', True, 'Add new affiliation...')
    if request.method == 'POST':
        # set user to authenticated user
        user = User.objects.get(id=request.user.id)
        user_form = UserForm(request.POST, request.FILES)
        if user_form.is_valid():
            if user_form.cleaned_data["photo"]:
                user.photo = user_form.cleaned_data["photo"]                
            user.save()
            logger.debug(user.__dict__)

            return render(request, 'edit_user.html', {'user_form': user_form, 'user': user, 'updated': True,
                'oep_affiliations':oep_affiliations, 'oep_projects':oep_projects, 'oep_publications':oep_publications, })
            #return HttpResponseRedirect('/data/user/' + str(user.id))
        else:
            logger.debug( "not valid")
            logger.debug( user_form.errors.as_data())
    else:
        user = User.objects.get(id=request.user.id)
        user_form = UserForm(instance=user)
        logger.debug('user_form1')
        logger.debug(user_form)
    
    return render(request, 'edit_user.html', {'user_form': user_form, 'user': user,
        'oep_affiliations': oep_affiliations, 'oep_projects':oep_projects, 'oep_publications':oep_publications, })

#    user = User.objects.get(id=request.user.id)
#    return render(request, 'data/edit_user.html', {'user': user})

#@csrf_exempt
def user_list_ajax(request):
    queryset = User.objects.all()
    return render(request, 'user_list_ajax.html', {'queryset':queryset,})

def user_detail_ajax(request, pk):
    user = User.objects.get( id=pk )
    return render(request, 'user_detail_ajax.html', {'user':user,})

#def user_list(request, page=1):
#    countries = Country.objects.all().exclude(public=False)
#    return render(request, 'user_list.html', {'pagetitle':'Researchers', 'countries':countries, 'page':page,})

@login_required(login_url='login')
def user_list(request, page=1):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    queryset = User.objects.filter(access='PUB').exclude(is_active=False).exclude(first_name='').exclude(last_name='')
    countries_tags = None
    country = None
    pagination = False
    if request.GET:
        try:
            country = request.GET['country']
            country_obj = Country.objects.filter(slug=country).exclude(public=False)
            queryset = queryset.filter(countries=country_obj)
        except:
            pass
    #if request.POST:
    #    countries_tags = request.POST.getlist('countries')
    #    print(countries_tags)
    #    for tag in countries_tags:
    #        country_obj = Country.objects.filter(name=tag).exclude(public=False)[0]
    #        queryset = queryset.filter(countries=country_obj)
    #        print("filter", tag)
    #print(queryset.count())

    # if more then 24 users activate pagination
    if queryset.count() >= 24:
        paginator = Paginator(queryset, 25) # Show 25 contacts per page
        pagination = True
        try:
            users = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            users = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            users = paginator.page(paginator.num_pages)
    else:
        users = queryset

    return render(request, 'user_list.html', {'pagination': pagination,'users': users, 'pagetitle':'Researchers', 'country':country, 'countries': countries, 'countries_tags': countries_tags})

#def user_list_country(request, country):
#    countries = Country.objects.all()
#    country_obj = Country.objects.get(slug=country)
#    queryset = User.objects.filter(public=True).filter(countries=country_obj)
#    return render(request, 'data/user_list.html', {'queryset':queryset, 'pagetitle': 'Researchers', 'country':country_obj,'countries' : countries,})

def user_detail(request, uuid):
    try:
        if request.user.is_authenticated:
            user = User.objects.filter(uuid=uuid).exclude(access='HID')[0]
        else:
            user = User.objects.filter(uuid=uuid).filter(access='PUB')[0]
    except:
        raise Http404("Page not found.")    
    return render(request, 'user_detail.html', {'user' : user, })

def project_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
            if request.user.is_authenticated:
                projects = Project.objects.exclude(access='HID').filter(countries=country_obj)
            else:
                projects = Project.objects.filter(access='PUB').filter(countries=country_obj)
        except IndexError:
            if request.user.is_authenticated:
                projects = Project.objects.exclude(access='HID')
            else:
                projects = Project.objects.filter(access='PUB')
    else:
        if request.user.is_authenticated:
            projects = Project.objects.exclude(access='HID')
        else:
            projects = Project.objects.filter(access='PUB')
    return render(request, 'data/project_list.html', {'country':country_obj, 'countries':countries, 'projects':projects, 'pagetitle':'Projects'})

def protest_list(request):
    countries = Country.objects.all().exclude(public=False)
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country).exclude(public=False)[0]
            pes = ProtestEvent.objects.filter(country=country_obj)
        except IndexError:
            pes = ProtestEvent.objects.all()
    else:
        pes = ProtestEvent.objects.all()
    return render(request, 'data/protest_list.html', {'country':country_obj, 'countries':countries, 'pes':pes, 'pagetitle':'Protests'})

def protest_detail(request, pk):
    try:
        pe = ProtestEvent.objects.get(id=pk)
    except:
        raise Http404("Page not found.")
    return render(request, 'data/protest_detail.html', {'pe' : pe,})

def project_list_country(request, country):
    country_obj = Country.objects.get(slug=country)
    projects = Project.objects.filter(countries=country_obj)
    return render(request, 'data/project_list.html', {'projects':projects, 'pagetitle':'Projects'})

def project_detail(request, pk):
    try:
        if request.user.is_authenticated:
            project = Project.objects.filter(id=pk).exclude(access='HID')[0]
        else:
            project = Project.objects.filter(id=pk).filter(access='PUB')[0]
    except:
        raise Http404("Page not found.")
    return render(request, 'data/project_detail.html', {'project' : project,})

def category_list(request):
    categories = Category.objects.all()
    url_detail = 'category_detail'
    return render(request, 'category_list.html', {'objects':categories, 'pagetitle':'Categories'})

def category_detail(request, pk):
    countries = Country.objects.all().exclude(public=False)
    category = get_object_or_404(Category, id=pk)
    return render(request, 'category_detail.html', {'object': category, 'countries' : countries})

@login_required(login_url='login')
def documentation_list(request):
    docu_general = Documentation.objects.filter(public=True, docu_type='GEN').order_by('title')
    docu_best_practices = Documentation.objects.filter(public=True, docu_type='BPR').order_by('title')
    docu_legal = Documentation.objects.filter(public=True, docu_type='LEG').order_by('title')
    object_url = 'documentation_detail'
    return render(request, 'category_list.html', {'docu_general': docu_general, 'docu_best_practices': docu_best_practices,
                                                  'docu_legal': docu_legal, 'pagetitle':'Rules and Best Practices', 'object_url':object_url,})

def documentation_detail(request, pk=None, slug=None):
    if pk:
        documentation = get_object_or_404(Documentation, id=pk)
    if slug:
        documentation = get_object_or_404(Documentation, slug=slug)

    return render(request, 'category_detail.html', {'object' : documentation,})


@login_required(login_url='login')
def institution_list(request):
    countries = Country.objects.all()
    country_obj = None
    if request.GET:
        country = request.GET['country']
        try:
            country_obj = Country.objects.filter(slug=country)[0]
            institutions = Institution.objects.filter(public=True).filter(country=country_obj)
        except IndexError:
            institutions = Institution.objects.filter(public=True)
    else:
        institutions = Institution.objects.filter(public=True)
    return render(request, 'institution_list.html', {'country':country_obj, 'countries':countries, 'institutions':institutions, 'pagetitle':'Institutions'})

@login_required(login_url='login')
def institution_detail(request, pk):
    institution = get_object_or_404(Institution, id=pk)

    return render(request, 'institution_detail.html', {'institution' : institution,})

@login_required(login_url='login')
def logout_view(request):
    logout(request)



"""
not used
"""

class TopicList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Topic.objects.all()
    serializer_class = TopicSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class TopicDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Topic.objects.all()
    serializer_class = TopicSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

class DataSetList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = DataSet.objects.all()
    serializer_class = DataSetSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class DataSetDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = DataSet.objects.all()
    serializer_class = DataSetSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class InstitutionList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Institution.objects.all()
    serializer_class = InstitutionSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class InstitutionDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Institution.objects.all()
    serializer_class = InstitutionSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class ProjectList(mixins.ListModelMixin, generics.GenericAPIView):
    queryset = Project.objects.all()
    serializer_class = ProjectSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

class ProjectDetail(mixins.RetrieveModelMixin, generics.GenericAPIView):
    queryset = Project.objects.all()
    serializer_class = ProjectSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

class UserList(mixins.ListModelMixin,
                        #mixins.CreateModelMixin,
                        generics.GenericAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

    #def post(self, request, *args, **kwargs):
    #    return self.create(request, *args, **kwargs)

class UserDetail(mixins.RetrieveModelMixin,
                        #mixins.UpdateModelMixin,
                        #mixins.DestroyModelMixin,
                        generics.GenericAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    #def put(self, request, *args, **kwargs):
    #    return self.update(request, *args, **kwargs)

    #def delete(self, request, *args, **kwargs):
    #    return self.destroy(request, *args, **kwargs)

